# (c) Copyright 2009 Synapse Wireless, Inc.


import logging
log = logging.getLogger("snaplib.%s" % (__name__))


class ConnectionManager(object):
    def __init__(self, process_conn):
        self.process_conn = process_conn
        self._open = False
        self._type = ''
        self._port = 0
        self._portstr = ''
        self._baurdrate = 0
        self.closeRegistry = []
        self.openRegistry = []
        self.pktTxRegistry = []

    def close(self):
        self.process_conn.send(("close", ()))

    def flushReadBuf(self):
        self.process_conn.send(("flushReadBuf", ()))

    def isOpen(self):
        return self._open

    def sendData(self, string):
        self.process_conn.send(("sendData", (string,)))

    def setOutputType(self, type, port=0):
        self._type = type
        self.process_conn.send(("setOutputType", (type, port)))

    def getBaudrate(self):
        return self._baurdrate

    def getPortstr(self):
        return self._portstr

    def onDeviceRemoval(self):
        self.process_conn.send(("onDeviceRemoval", ()))

    def OnPktTx(self, lastSendOk):
        for cb in self.pktTxRegistry:
            try:
                cb(lastSendOk)
            except:
                log.exception("An error occurred while notifiying a callback about pkt tx:")

    def getOutputType(self):
        return self._type

    def onPyserialClose(self):
        self._open = False
        for cb in self.closeRegistry:
            try:
                cb()
            except:
                log.exception("An error occurred while notifiying a callback about the connection closing:")

    def onPyserialOpen(self, portstr, baurdrate):
        self._portstr = portstr
        self._baurdrate = baurdrate
        self._open = True

        for cb in self.openRegistry:
            try:
                cb()
            except:
                log.exception("An error occurred while notifiying a callback about the connection opening:")

    def registerCloseCallback(self, callback):
        """
        Register for callbacks when the the PyserialDriver is closed

        callback -- A callable object with a no parameters
        """
        if callable(callback):
            self.closeRegistry.append(callback)
  
    def registerOpenCallback(self, callback):
        """
        Register for callbacks when the the PyserialDriver is opened

        callback -- A callable object with a no parameters
        """
        if callable(callback):
            self.openRegistry.append(callback)

    def registerOnPktTx(self, callback):
        if callable(callback):
            self.pktTxRegistry.append(callback)

    baudrate = property(getBaudrate)
    portstr = property(getPortstr)
