# (c) Copyright 2009, Synapse Wireless, Inc.


import os
import sys
import time
import logging, logging.handlers
import threading

import PyserialDriver
if not hasattr(sys, 'frozen'):
    from .snaplib import PacketSerialProtocol, SnapConstantsHelper, MinimalMeshHandler
    from .snaplib import Globleton #NOTE: This these are not sharped with other process
else:
    #Not sure why pyinstaller has a problem with relative import on second process
    from snaplib import PacketSerialProtocol, SnapConstantsHelper, MinimalMeshHandler
    from snaplib import Globleton #NOTE: This these are not sharped with other process

from multiprocessing import queues
import multiprocessing
import timeit
import select


log = multiprocessing.get_logger()
LOG_NAME = 'SerialTrace'

PROCESS_NAME = 'Serial'


assert False # This module is not guarenteed to be kept up to date


class SerialProcess(object):
    def __init__(self, conn1, conn2, verboseLogging, dllPath, handle_mesh=True):
        self.pollCallbacks = []
        self.conn = conn2
        self.our_conn = conn1
        self.myAddresses = []
        self.sendQueue = []
        self.handle_mesh = handle_mesh

        self.pyserialDriver = PyserialDriver.PyserialWrapper(dllPath=dllPath)
        self.pyserialDriver.registerOpenCallback(self.onPyserialOpen)
        self.pyserialDriver.registerCloseCallback(self.onPyserialClose)
        self.pktSer = PacketSerialProtocol.PacketSerialDriver(self.pyserialDriver, self)
        self.pktSer.verboseLogging = verboseLogging

        self.pktSer.registerCallback("rx", self.onPktSerRx)
        self.pktSer.registerCallback("tx", self.onPktSerTx)
 
        self.read_thread = threading.Thread()

    def addAddress(self, addr):
        if addr not in self.myAddresses:
            self.myAddresses.append(addr)

    def close(self):
        log.debug("Got close request")
        if self.pyserialDriver.isOpen and self.read_thread.isAlive():
            self.pyserialDriver.read_looping =  False
            #Stop read loop and wait for thread to die
            self.read_thread.join()
        self.pyserialDriver.close()

    def flushReadBuf(self):
        self.pyserialDriver.flushReadBuf()

    def loop(self):
        while True:
            log.debug("loop1: %f" % (timeit.default_timer()))
            try:
                log.debug("loop2: %f" % (timeit.default_timer()))
                #if self.conn.poll(0.001):
                r, w, e = select.select([self.conn], [], [], 0.001)
                if self.conn.poll(0.001):
                    log.debug("loop3: %f" % (timeit.default_timer()))
                    (func_name, args) = self.conn.recv()
                    log.debug("loop4: %f" % (timeit.default_timer()))
                    assert isinstance(func_name, str)
                    func = getattr(self, func_name, 0)
                    if func:
                        func(*args)
            except IOError:
                #Occurrs when main process is killed
                log.debug("shutting down process")
                self.pyserialDriver.close()
                self.pyserialDriver.read_looping = False
                self.read_thread.join()
                break

            log.debug("loop5: %f" % (timeit.default_timer()))
            self.pyserialDriver.readPoll()
            log.debug("loop6: %f" % (timeit.default_timer()))
            for cb in self.pollCallbacks:
                cb()
            log.debug("loop7: %f" % (timeit.default_timer()))
            self.pyserialDriver.writePoll()
            log.debug("loop8: %f" % (timeit.default_timer()))
        log.debug("Serial Process Exited")

    def onDeviceRemoval(self):
        self.pyserialDriver.onDeviceRemoval()

    def onPktSerRx(self, pkt):
        pid = ord(pkt[SnapConstantsHelper.RPC_PID_OFS])
        if self.handle_mesh and ((pid == SnapConstantsHelper.PID_MESH_PTP) or (pid == SnapConstantsHelper.PID_MESH_BCAST)):
            reply_desc = MinimalMeshHandler.processMeshPacket(pkt, self.myAddresses)
            if reply_desc:
                self.our_conn.send(("sendData", (reply_desc.encoded_data,)))
        else:
            self.conn.send(("handlePktRx", (pkt,)))
        #self.rxQueue.put_nowait(pkt)

    def onPktSerTx(self, lastSendOk, tell=True):
        #log.debug("process OnPktTx")
        if tell:
            self.conn.send(("OnPktTx", (lastSendOk,)))
        try:
            self.pktSer.sendData(self.sendQueue.pop(0))
        except IndexError:
            pass

    def onPyserialClose(self):
        self.conn.send(("onPyserialClose", ()))
        self.sendQueue = []

    def onPyserialOpen(self, portstr, baud):
        self.conn.send(("onPyserialOpen", (portstr, baud)))

    def printTest(self, text1, text2):
        print text1
        print text2
        sys.stdout.flush()

    def registerPollCallback(self, callback):
        """
        Register for callbacks for object who want to get polled

        callback -- A callable object with a no parameters
        """
        assert callable(callback)
        if callable(callback):
            self.pollCallbacks.append(callback)

    def removeAddress(self, addr):
        if addr in self.myAddresses:
            self.myAddresses.remove(addr)

    def sendData(self, string):
        if self.pktSer.ready_to_send():
            self.pktSer.sendData(string)
        else:
            self.sendQueue.append(string)

    def setOutputType(self, type, port=0):
        try:
            self.pyserialDriver.setOutputType(type, port, True)
            assert not self.read_thread.isAlive()
            self.read_thread = threading.Thread(target=self.pyserialDriver.readLoop)
            self.read_thread.setDaemon(True)
            self.read_thread.start()
        except PyserialDriver.SerialOpenException, ex:
            self.conn.send(("openException", (ex,)))
        #self.conn.send(("openConnection", ()))


def initProcess(conn1, conn2, verboseLogging=__debug__, dllPath=os.path.curdir, handle_mesh=True, logFilePath=os.curdir, logFileLevel=logging.INFO):
    if __debug__:
        if not sys.platform.startswith("win"):
            def handler(sig, frame):
                print "SIGTRAP"
            import signal
            signal.signal(signal.SIGTRAP, handler)

        #Allows us to launch another copy of Wing so we can debug this process
        os.environ.pop("WINGDB_ACTIVE", 0)
        import wingdbstub

        # Log to stderr
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s:%(msecs)03d %(levelname)-8s %(name)-8s %(message)s', datefmt='%H:%M:%S')
    else:
        # Log to rotating files
        rotatingFileLogHandler = logging.handlers.RotatingFileHandler(os.path.join(logFilePath, LOG_NAME), maxBytes=500000, backupCount=5)
        logFormat = logging.Formatter('%(asctime)s:%(msecs)03d %(levelname)-8s %(name)-8s %(message)s', datefmt='%H:%M:%S')
        rotatingFileLogHandler.setFormatter(logFormat)
        rotatingFileLogHandler.setLevel(logFileLevel)
        log.addHandler(rotatingFileLogHandler)

    debug = ''
    if __debug__:
        debug = "Debug"
    log.info('******** %s %s Serial Log - %s %s ********' % (Globleton.company, Globleton.product, Globleton.version, debug))

    SerialProcess(conn1, conn2, verboseLogging, dllPath, handle_mesh).loop()


if __name__ == '__main__' and not hasattr(sys, 'frozen'):
    multiprocessing.freeze_support()

    (conn1, conn2) = multiprocessing.Pipe(True)
    txQueue = queues.Queue()
    rxQueue = queues.Queue()

    p = multiprocessing.Process(target=initProcess, args=(conn1, conn2))
    p.start()
    conn1.send(("printTest", ("test", "test2")))
    conn1.send(("setOutputType", (2, 2)))
    conn1.send(("setOutputType", (2, 0)))
    while True:
        try:
            if conn1.poll(0.1):
                (func_name, args) = conn1.recv()
                print func_name, args
                #sys.stdout.flush()
        except IOError:
            break
