# (c) Copyright 2007-2010, Synapse Wireless, Inc.
"""Automatically pickle an object"""


import sys
import logging
log = logging.getLogger('snap.AutoSaver')

import os
import cPickle


# Make it possible to load nvparams.dat files created with earlier versions (name changes)
if sys.modules.has_key('snapconnect.AutoPickler'):
    sys.modules['Snap.AutoPickler'] = sys.modules['snapconnect.AutoPickler']
    sys.modules['snapcom.AutoPickler'] = sys.modules['snapconnect.AutoPickler']

def loadSaver(filepath):
    """
    Given the provided pickle file open it and return the saved object
    
    filepath : The full path and file name of the saved pickle file
    """
    openFile = open(filepath, 'rb')
    prevSaver = cPickle.load(openFile)
    prevSaver._auto_save_file_path = filepath
    openFile.close()
    return prevSaver


class AutoSaver(object):
    """An object that automatically pickles itself when attributes are stored or modified"""
    def __init__(self, filepath):
        """
        Initializes an AutoSaver instance
        
        filepath : The full path and file name of the saved pickle file
        """
        if not os.path.isdir(os.path.dirname(os.path.abspath(filepath))):
            os.mkdir(os.path.abspath(os.path.dirname(filepath)))
        self._auto_save_file_path = filepath

    def __setattr__(self, name, value):
        """Overrides the default object implementation so we know when to save"""
        #See if we can pickle this first
        cPickle.dumps(value)
        object.__setattr__(self, name, value)
        self.__autosave__()

    def __delattr__(self, name):
        """Overrides the default object implementation so we know when something is deleted and we should save"""
        object.__delattr__(self, name)
        self.__autosave__()

    def __autosave__(self):
        """Save this instance of the object"""
        try:
            saveFile = open(self._auto_save_file_path, 'wb')
            cPickle.dump(self, saveFile)
            saveFile.close()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            log.exception('Exception:')
            raise


if __name__ == '__main__':
    import random

    logging.basicConfig(level=logging.DEBUG, format='%(asctime)s:%(msecs)03d %(levelname)-8s %(name)-8s %(message)s', datefmt='%H:%M:%S')
    testString = ''.join(random.sample('abcdefghijklmnopqrstuvwxyz', 25))
    testString2 = ''.join(random.sample('abcdefghijklmnopqrstuvwxyz', 25))

    newSaver = AutoSaver(os.path.join(os.getcwd(), 'AutoSaver.dat'))
    newSaver.test = testString
    newSaver.delTest = ''.join(random.sample('abcdefghijklmnopqrstuvwxyz', 25))
    del newSaver.delTest
    del newSaver

    oldSaver = loadSaver(os.path.join(os.getcwd(), 'AutoSaver.dat'))
    if oldSaver.test == testString:
        log.info('Strings match: %s' % oldSaver.test)
    else:
        log.error('Strings do NOT match! oldSaver.test=%s testString=%s' % (oldSaver.test, testString))
    if hasattr(oldSaver, 'delTest'):
        log.error('delTest was NOT removed')
    else:
        log.info('delTest was removed')

    oldSaver.test2 = testString2

    oldSaver2 = loadSaver(os.path.join(os.getcwd(), 'AutoSaver.dat'))
    if oldSaver2.test2 == testString2:
        log.info('Strings match: %s' % oldSaver2.test2)
    else:
        log.error('Strings do NOT match! oldSaver.test=%s testString=%s' % (oldSaver2.test2, testString2))
