# (c) Copyright 2007-2010, Synapse Wireless, Inc.


import binascii
import cPickle
import math

import LicenseManager
from snaplib.ppcrypto import ARC4


def save_license(license, filename=LicenseManager.default_filename):
    """
    Encrypts and saves the provided license object
    
    license : The LicenseManager.License object to save
    filename : The full path and filename to use when saving
    """
    saveFile = open(filename, 'wb')
    saveImage = cPickle.dumps(license, 2)
    encoder = ARC4(LicenseManager.__key+str(math.pi))
    c = encoder.encrypt(saveImage)
    saveFile.write(c)
    saveFile.close()


if __name__ == '__main__':
    import optparse

    # Parse command-line options
    parser = optparse.OptionParser()
    parser.add_option("-d",
                      type="int", dest="valid_days", metavar="DAYS", default=30,
                      help="Number of days license is valid for")

    parser.add_option("-l",
                      type="string", dest="licensee", default="Demo Licensee",
                      help="Licensee name", metavar="LICENSEE")

    parser.add_option("-o",
                      type="string", dest="outputFile", default=LicenseManager.default_filename,
                      help="Output file name", metavar="FILE")

    parser.add_option("-p",
                      action="store_true", dest="permanent", default=False,
                      help="Permanent license", metavar="PERMANENT")

    parser.add_option("-a",
                      type="string", dest="addr",
                      help="Network Address", metavar="ADDR")

    parser.add_option("-b",
                      type="string", dest="last_addr", default=None,
                      help="Last Network Address", metavar="LASTADDR")

    (options, args) = parser.parse_args()

    addr = [binascii.unhexlify(options.addr)]
    last_addr = options.last_addr
    if last_addr is not None:
        last_addr = binascii.unhexlify(options.last_addr)
        if int(binascii.hexlify(last_addr), 16) > int(binascii.hexlify(addr[0]), 16):
            addr = [binascii.unhexlify("%06X" % (x)) for x in range(int(binascii.hexlify(addr[0]), 16), int(binascii.hexlify(last_addr), 16)+1)]
    lic = LicenseManager.License(tuple(addr), options.permanent, options.valid_days, options.licensee)

    save_license(lic, options.outputFile)
    LicenseManager.check_license(options.outputFile)
