# (c) Copyright 2007-2010, Synapse Wireless, Inc.


from snaplib.ppcrypto import ARC4

import cPickle
import datetime
import math
import os
import binascii
import uuid
import sys

import LicenseManager # Yes, import yourself
if sys.modules.has_key('snapcom.LicenseManager'):
    sys.modules['LicenseManager'] = sys.modules['snapcom.LicenseManager']
if sys.modules.has_key('Snap.LicenseManager'):
    sys.modules['LicenseManager'] = sys.modules['Snap.LicenseManager']
if sys.modules.has_key('snapconnect.LicenseManager'):
    sys.modules['LicenseManager'] = sys.modules['snapconnect.LicenseManager']


__key='Synapse'
default_filename = 'License.dat'
VERSION = 2
SNAP_MAC = "001C2C"


class LicenseError(Exception):
    """Thrown when an invalid license is found"""
    pass


class License(object):
    """A generic license object to use in products to enforce licensing restrictions"""
    def __init__(self, addr, permanent=False, valid_days=60, licensee=None):
        """
        Initializes an instance of a License object
        
        addr : The SNAP address(es) this license is for
        permanent : Does this license expire?
        valid_days : How many days this license is valid for after creation
        licensee : A description of who this license is for
        """
        self._inCtor = True

        self.permanent = permanent
        self.debug = __debug__
        self.create_date = datetime.datetime.utcnow()
        self.licensee = licensee
        self.version = VERSION
        if permanent:
            self.valid_days = datetime.timedelta().max
            if self.licensee is None:
                self.licensee = "Permanent Licensee"
        else:
            self.valid_days = datetime.timedelta(days=valid_days)
            if self.licensee is None:
                self.licensee = "Demo Licensee"
        if not isinstance(addr, tuple):
            addr = (addr, )
        self.addr = addr

        self._inCtor = False

    def __setattr__(self, name, val):
        """After __init__, this class is read-only"""
        if name == '_inCtor' or self._inCtor:
            object.__setattr__(self, name, val)
        else:
            pass

    def __str__(self):
        if self.permanent:
            if len(self.addr) > 1:
                other_addrs = " and %d other addresses" % (len(self.addr) - 1)
            else:
                other_addrs = ""
            return "Permanent license created on %s for %s%s" % (self.create_date, binascii.hexlify(self.addr[0]), other_addrs)
        return "Demo license created on %s and valid for %i days for address %s" % (self.create_date,
                                                                                    self.valid_days.days,
                                                                                    binascii.hexlify(self.addr[0]))


def check_license(filename=default_filename):
    """
    Open the provided license filename and return a valid license if possible
    
    filename : The full path and filename of the license file
    """
    if filename is None:
        filename = default_filename
    try:
        license_ = load_license(filename)
        if __debug__:
            print license_
        assert isinstance(license_, LicenseManager.License)
        if not license_.permanent:
            time_left = datetime.datetime.utcnow()-license_.create_date
            if time_left > license_.valid_days:
                raise LicenseError("Your %i day license has expired" % (license_.valid_days.days))
            elif license_.create_date > datetime.datetime.utcnow():
                raise LicenseError("Your license is invalid, it was created in the future")
        return license_
    except LicenseError, le:
        raise le
    except:
        mac_addr = "%012X" % (uuid.getnode())
        if mac_addr.startswith(SNAP_MAC):
            return License(binascii.unhexlify(mac_addr[6:]), permanent=True, licensee="SNAP Licensed Hardware")
        raise LicenseError("Unable to load license file %s" % (os.path.normpath(filename)))

def load_license(filename=default_filename):
    """
    Open and decript the license

    filename : The full path and filename of the license file
    """
    saveFile = open(filename, 'rb')
    saveImageCrypt = saveFile.read()
    encoder = ARC4(__key+str(math.pi))
    saveImage = encoder.decrypt(saveImageCrypt)
    license_ = cPickle.loads(saveImage)
    saveFile.close()

    return license_
