# (c) Copyright 2007, Synapse
"""SNAPconnect responses to clients"""
__docformat__ = "plaintext en"
# Disable warning about docstring under variable names
# pylint: disable-msg=W0105


from apy import xmlrpclib


SYNAPSE_XMLRPC_MULTIPE_WAITS_FAULTCODE = 100
SYNAPSE_XMLRPC_CONNECTION_CLOSED_FAULTCODE = 200
SYNAPSE_XMLRPC_NO_CONNECTION_FAULTCODE = 201


class EventResponse(object):
    """An XML-RPC structure that is returned on a node event"""
    def __init__(self, netAddr, method, timestamp, *args):
        self.netAddr = xmlrpclib.Binary(netAddr)
        """The SNAP network address this RPC originated from"""
        self.methodName = method
        """The string representing the method to call"""
        self.parameters = []
        """An array of parameters for the method specified by methodName
           Note, all "string" parameters are converted to XML-RPC base64 type (binary)
           This was done to avoid automatic character translation by XML-RPC library functions
        """

        for arg in args:
            if isinstance(arg, str):
                arg = xmlrpclib.Binary(arg)
            self.parameters.append(arg)

        self.parameters = tuple(self.parameters)
        self.timestamp = timestamp
        """A floating point number expressed in seconds since the epoch, in UTC, of when the event was received"""


class InfoResponse(object):
    """An XML-RPC structure that is returned from the gatewayInfo method"""
    def __init__(self, version, connected, portType, portNum):
        self.version = version
        """The version of SNAPconnect"""
        self.connected = connected
        """Whether SNAPconnect is connected to a bridge node"""
        self.portType = portType
        """The type of port currently in use for the serial connection (RS232=1, USB=2)
           A value of "None" is returned if SNAPconnect does not have a serial connection"""
        self.portNum = portNum
        """The port number of the specified portType starting at 1
           A value of "None" is returned if SNAPconnect does not have a serial connection
           For USB ports, a portNum of 1 is associated with USB0 on a PC
           For RS-232 ports, a portNum of 1 is associated with COM1 on a PC
        """
