# (c) Copyright 2007, Synapse
"""Classes to support having a tray icon for SNAPconnect"""
__docformat__ = "plaintext en"


import logging, logging.handlers, cStringIO, sys, webbrowser, Queue
log = logging.getLogger('SNAPconnectApp')

import wx
from synapseGuiLib import WndProcHookMixinCtypes, Pbt
if __debug__:
    import threading


USBX_GUID = "{3c5e1462-5695-4e18-876b-f3f3d08aaf18}"


#Based off the code in the wxPython demo Main.py
class SNAPconnectTaskBarIcon(wx.TaskBarIcon):
    def __init__(self, frame, task_queue, done_queue, showStatusPage, version, portNum):
        wx.TaskBarIcon.__init__(self)
        self.frame = frame
        self.showStatusPage = showStatusPage
        self.portNum = portNum
        self.task_queue = task_queue
        assert isinstance(self.task_queue, Queue.Queue)
        self.done_queue = done_queue
        assert isinstance(self.done_queue, Queue.Queue)

        # Set the image
        stream = cStringIO.StringIO( self.getsynapseIconData() )
        img = wx.ImageFromStream(stream)
        icon = self.MakeIcon(img)
        self.SetIcon(icon, "SNAPconnect %s" % (version))

    def CreatePopupMenu(self):
        """
        This method is called by the base class when it needs to popup
        the menu for the default EVT_RIGHT_DOWN event.  Just create
        the menu how you want it and return it from this function,
        the base class takes care of the rest.
        """
        self.task_queue.put('getSerialConnInfo')

        menu = wx.Menu()

        if self.showStatusPage:
            statuspage = menu.Append(wx.ID_ANY, "Open Status Page")
            self.Bind(wx.EVT_MENU, self.handleStatusPage, statuspage)
            menu.AppendSeparator()

        port = menu.Append(wx.ID_ANY, "Listening on %i" % (self.portNum))
        port.Enable(False)
        try:
            result = self.done_queue.get(block=True, timeout=0.5)
            conn = menu.Append(wx.ID_ANY, "Serial Connections:")
            conn.Enable(False)
            for desc in result:
                conn_info = menu.Append(wx.ID_ANY, "  %s" % (desc))
                conn_info.Enable(False)
        except Queue.Empty:
            pass
        menu.AppendSeparator()

        menu.Append(wx.ID_EXIT, "Exit")

        return menu

    def handleStatusPage(self, event):
        webbrowser.open("http://localhost:%i/status" % (self.portNum), 2, 1)
        event.Skip()

    def MakeIcon(self, img):
        """
        The various platforms have different requirements for the icon size...
        """
        if "wxMSW" in wx.PlatformInfo:
            img = img.Scale(16, 16)
        elif "wxGTK" in wx.PlatformInfo:
            img = img.Scale(22, 22)
        # wxMac can be any size upto 128x128, so leave the source img alone....
        icon = wx.IconFromBitmap(img.ConvertToBitmap())
        return icon

    def OnTaskBarClose(self, event):
        self.RemoveIcon()
        event.Skip()

    @staticmethod
    def getsynapseIconData():
        return \
'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00 \x00\x00\x00 \x08\x06\x00\
\x00\x00szz\xf4\x00\x00\x00\x04sBIT\x08\x08\x08\x08|\x08d\x88\x00\x00\x06\
\x12IDATX\x85\xbd\x97\xcdOS]\x1a\xc0\x7f\xe7~\x94\xde\xb6Nm\xf9H)}\x0b\x11\
\x91\xc1J\x8d\x8a\n&j$\xba\x9a\x05\xee\x8c1\xfe\x15&\xb8t5\x89\x7f\x80\x89\
\x9bI\x8c.\\\x18c\xdc:\x90\x91\x85\t~\xa4.\x88HdR(Nq\nH\x0b\xb4\xbd\xb4\xbd\
\xbd\xf7\xce\x02\xb9R\n\xbc\xf0\xbef\x9e\xa4M\xee=\xe79\xcf\xef\xf9\xb8\xcf9\
G\x006\xff\x07\x11B`\xdb\xf5\xa6\xa4\xed/\x82\xc1 \xad\xad\xad\x08!~\x99\xe1\
H$BOO\x0f\x8a\xa2\xec\r \xcb2CCC\xdc\xb9s\x87p8\xfcK\x00\xbc^/\xb7o\xdf\xe6\
\xee\xdd\xbb\xb4\xb4\xb4\xec\r\x10\x8dF\x19\x1a\x1ab``\x00\x9f\xcf\xf7\xa7\
\x0co\x8d\xe0\xa1C\x87\xe8\xef\xef\xa7\xbf\xbf\x7fw\x00Y\x969{\xf6,\x9d\x9d\
\x9d\x8c\x8e\x8e\x92N\xa7\xffT\x1a6\xf3\xad\xeb:ccc\x98\xa6\xc9\x85\x0b\x17P\
Uug\x00\xbf\xdf\xcf\xe9\xd3\xa7\x01\x18\x19\x19A\xd7uTU\xa5\xb7\xb7\x97\xde\
\xde^dY\xde\xb7\xf1\x86\x86\x06\x9a\x9a\x9ap\xbb\xddX\x96E"\x91`zz\x9a\x93\'\
Or\xe4\xc8\x11\xe0g\x84\x1c\x80`0H4\x1a%\x99L\xf2\xf5\xebW\x00"\x91\x08\xc3\
\xc3\xc3\x0c\x0f\x0f\xd3\xde\xde\xbe/\xe3\xb2,s\xee\xdc9\xee\xdf\xbf\xcf\xe0\
\xe0 \xaa\xaa\xb2\xb6\xb6\xc6\xf8\xf88\x8d\x8d\x8dtww#I\x92\x13!\t@\x92$B\
\xa1\x10\xd1h\x94\x99\x99\x19\xf2\xf9<\xaa\xaar\xf9\xf2e\xe2\xf18\xa9T\x8a\
\xb5\xb5\xb5}\x01H\x92\x84\xa6i\x9c8q\x82\x9b7o\xd2\xd6\xd6\x86eY|\xf9\xf2\
\x05\xb7\xdb\xcd\xd1\xa3Gihh\xf89\x1f@Q\x14\xc2\xe10\x81@\x80t:M\xb9\\&\x18\
\x0c288H6\x9b\xe5\xe5\xcb\x97|\xff\xfe\xbd\xc6\x88\xdf\xef\xa7\xb9\xb9\xb9.\
\xa7\x86a\x90H$x\xf3\xe6\r\xf1x\x9cX,\x86\x10\x82\xf9\xf9y\xca\xe52\xe1p\x18\
\xaf\xd7[\x0b\xa0\xaa*\x81@\x00\x80\x85\x85\x05\x0c\xc3\xa0\xb5\xb5\x95\x8e\
\x8e\x0efggI\xa5R\x8e\x82\x10\x82\x81\x81\x01\x1e>|\xc8\x83\x07\x0f\xb8q\xe3\
\x86\xa3\xbb)\xb9\\\x8e\xf7\xef\xdfS\xadV9~\xfc8\x8a\xa2\xb0\xb2\xb2B>\x9f\
\xc7\xef\xf7\xa3i\x9a3W\xd9\xcc\x9b\xc7\xe3\xc10\x0c\n\x85\x02\x96eq\xf8\xf0\
a4Mcii\t]\xd7\x1d\x85@ \xc0\xbd{\xf7\xb8r\xe5\n\x00\x9d\x9d\x9dd2\x19\xc6\
\xc6\xc6\xa8V\xab\x00X\x96E*\x95B\xd7u\x8e\x1d;\x86\xa6iT*\x15J\xa5\x12\x9a\
\xa6\xe1r\xb9j\x01\x84\x10NUn\x16\xc7f\xd5W\xab\xd5\x9a\x16\xda\xd5\xd5\xc5\
\xa9S\xa7\x9c\xae\xd6\xde\xdeN(\x14B\x92\xa4\x9av[*\x95\xb0,\x0b\xbf\xdf_\
\x93\xa6\xed\x9f\xb6\xb4Il\x18\x06\xb2,;t\xc5b\x11\xc30\xf0\xf9|5-4\x97\xcb\
\xb1\xbc\xbc\xec<\x17\x8bEt]\xc7\xb6\xed\x1aP\x97\xcb\x85\x10\x82|>\x8fa\x18\
\xa8\xaa\x8a\xcb\xe5\xa2R\xa98\x91r\x00\xaa\xd5*\x85B\x01EQ\x08\x06\x83(\x8a\
\xc2\xd2\xd2\x12\xd9l\x96H$Bcc\xa3\xa3077\xc7\xe3\xc7\x8fY^^&\x9b\xcd222\xc2\
\xe7\xcf\x9f1M\xb3\xc6\xcbH$\x82\xa6i$\x93IJ\xa5\x12~\xbf\x1f\x9f\xcf\xc7\
\xda\xda\x1a\xa5R\xa9\x16\xc00\x0c\x16\x17\x17\xa9T*\x84\xc3a\\.\x17\x99L\
\x86\x0f\x1f>\xd0\xd1\xd1A__\x1f\x92\xb4\xd12\xca\xe52\xcf\x9f?\'\x91H099\
\xc9\xd3\xa7OI&\x935\xde\xfb|>\xce\x9c9\x83\xc7\xe3q\xe0B\xa1\x10>\x9f\x8f\
\xc5\xc5E\x8a\xc5b-\x80i\x9a|\xfb\xf6\x8dL&C4\x1aE\xd34t]\xe7\xd5\xabW\xac\
\xaf\xafs\xeb\xd6-zzz\x9c\xfc\x15\x8bE\xf2\xf9<\x85B\x81\\.W\x13RUU\xe9\xeb\
\xeb\xe3\xe2\xc5\x8bLMM111\x81m\xdbtuuQ\xa9TH&\x93\xac\xaf\xaf\xd7\x02\xd8\
\xb6\xcd\xc2\xc2\x02\xc9d\x92X,F(\x14\xc2\xb6m&&&x\xf2\xe4\t\x0b\x0b\x0bu{\
\xf9f\xe1n/^\xb7\xdbMoo/\x00\x8f\x1e=bnn\x0e\xaf\xd7\xcb\xa5K\x97X^^\xaeK\
\x97S]\xab\xab\xab\xcc\xcc\xccp\xf5\xeaU\xe2\xf18\x93\x93\x93\xe8\xba\xce\
\xb3g\xcf\x9cF\xb4\xd3\x81b\xbb\x14\n\x05^\xbcxA"\x91`jj\n\xd34\x89\xc5btww\
\xf3\xf6\xed[R\xa9T\r\xb4\x03\xa0\xeb:\x89D\x82\xd5\xd5U\xae_\xbf\xce\xeb\
\xd7\xaf\x99\x9f\x9fgee\xe5w\x8dn\x15\xdb\xb6I\xa7\xd3\xa4\xd3i`\xa3\x1e\xae\
]\xbb\x86\xc7\xe3a||\xbc\xae\xa5K[\x15?~\xfc\xc8\xbbw\xef\xe8\xe8\xe8\xa0\
\xb5\xb5uO#\xa6ibY\x16\xa6i\xee\x19\x19\xb7\xdbMss3\xd3\xd3\xd3\x8c\x8d\x8d\
\xd5\x8d+\xe7\xcf\x9fw\x1eL\xd3$\x99L"\x84\xc0\xef\xf7\xb3ul\xab\x04\x02\x01\
g\xbb\xed\x8d\xc7\xd14\x0f\xecqt\xf8\xf4\xe9\x13\xff\xcddhii\xa1\xb1\xa9\xa9\
fL\x14\xb3Y\x07\xdf\xb6\x01Y\xc2\x12\x12\x92Y\xdd}M!\xe1\xf2j\x08\x04\x86\
\xaec\x1a\xd5\xdd\x01l\xb0\xa4\x1f\x05kY\xf5\x11\xb0\xfe\xf1\xb8~\xfd\r\xbd]\
\x8f\xcb\xc2\xe3A\\\xff\xdb\xc6\xe2\xafF\xb1fg\xd9\xc8\xa6\xfdCg+\x8d\xfd\
\xe3_ \xc4\x0f\'\xb7\x02\xe4\xfe~\x7fG\x80mN\xfc\x14\xcbB\xe9<\x82\xeb\xcaE\
\x10\x82\xe2\xe8\xbf(\xfds\x04\xccz\xefvs\xac\x06@\xb8\xd4\x9d\xe6\xee\ndW\
\xab\xc8\xe10rs3v\xb5\x8a\xd2\xd6\x86\xf0\xfa\xa0\\\xdaU\x7f\xb7\xb5\x00\x14\
\x0ez\xf0\xb4Aj\x0c \xfe\xe2\x83r\x059\x12Fh\x1av\xb9\\;i\x9fRw1\xd9\xdb\xb8\
\r\x82\r\xaf]\rH\x1e\x0fJ{\x14\xd9\xe7\xddb\xf2`\x17\xad\x83\x01\x00Bu\xa1\
\x1e\xff+\x08\x01\x8a\x82\xfc[\x04\xd1\xdc\xf4\x87\x8f\xf0\xf5w\xa5\xbd\xc4\
\xb6\xc1\xedBx<T\xa7\xff\x8dm\x83\x9d\xcf#\x1a\x1a\xd8\xc8\xae\xc5\xce\xa5\
\xb6\xbb\x88\xff\xb4\xfc\xb6\xff\xd9\xb6\r.\x155\x16C\xf2x\xc0\xb6\xb1+\x15\
\x8c\xd9\x14v.W\xff\x8d\xfdr\x80M\xd9\xb2\x9b!\xc4\xcf\xdf\x1f\x90\x83\xa5`S\
\x0epK\xfa=\xf9\x1f\r\xfc\x8d:#\xfc;\x17\x00\x00\x00\x00IEND\xaeB`\x82'

class SNAPconnectFrame(wx.Frame, WndProcHookMixinCtypes.WndProcHookMixin):
    def __init__(self, parent, pyserial_close_event, close_event, task_queue, done_queue, version, showStatusPage=False, portNum=8080):
        wx.Frame.__init__(self, parent, -1, "SNAPconnect")
        WndProcHookMixinCtypes.WndProcHookMixin.__init__(self)

        self.close_event = close_event
        assert isinstance(self.close_event, threading._Event)
        self.pyserial_close_event = pyserial_close_event
        assert isinstance(self.pyserial_close_event, threading._Event)

        if sys.platform.startswith("win"):
            self.hookWndProc()
            self.devNotifyHandle = self.registerDeviceNotification(guid=USBX_GUID, devicetype=WndProcHookMixinCtypes.DBT_DEVTYP_DEVICEINTERFACE)
            self.addMsgHandler(WndProcHookMixinCtypes.WM_DEVICECHANGE, self.onDeviceChange)
            self.addMsgHandler(WndProcHookMixinCtypes.WM_POWERBROADCAST, self.onPowerChange)

        try:
            self.tbicon = SNAPconnectTaskBarIcon(self, task_queue, done_queue, showStatusPage, version, portNum)
            self.tbicon.Bind(wx.EVT_MENU, self.OnClose, id=wx.ID_EXIT)
        except:
            self.tbicon = None
            #self.SetIcon(PortalImages.getsynapseIcon())

            #panel = wx.Panel(self)
            #sizer = wx.FlexGridSizer(cols=2, hgap=5, vgap=5)
            #panel.SetSizerAndFit(sizer)
            #btn = wx.Button(panel, label="Exit")
            #sizer.Add(wx.StaticText(panel, label=""))
            #sizer.Add(btn)

            self.CenterOnScreen()
            self.Show()

    def OnClose(self, event=None):
        # Disable "Used argument" since this does get used by wxPython
        # pylint: disable-msg=W0613
        self.tbicon.RemoveIcon()
        #self.tbicon.Destroy() #I thought this was required but it causes a crash now
        self.Close(True)
        self.close_event.set()

    def ThreadClose(self):
        """Schedule closing this frame from another thread"""
        wx.CallAfter(self.OnClose)

    def onDeviceChange(self, wParam, lParam):
        # Disable has no member
        # pylint: disable-msg=E1101
        if __debug__:
            print "WM_DEVICECHANGE [WPARAM:%i][LPARAM:%i]"%(wParam,lParam)

        if lParam:
            dbh = WndProcHookMixinCtypes.DEV_BROADCAST_HDR.from_address(lParam)
            if dbh.dbch_devicetype == WndProcHookMixinCtypes.DBT_DEVTYP_DEVICEINTERFACE:
                dbd = WndProcHookMixinCtypes.DEV_BROADCAST_DEVICEINTERFACE.from_address(lParam)
                if dbd.dbcc_name.upper().find('VID_10C4&PID_8382') > -1:
                    #print "Was Synapse USB Device"
                    if wParam == WndProcHookMixinCtypes.DBT_DEVICEARRIVAL:
                        log.info("A Synapse USB Device was Discovered")
                    elif wParam == WndProcHookMixinCtypes.DBT_DEVICEREMOVECOMPLETE:
                        log.info("A Synapse USB Device was Removed")
                        self.pyserial_close_event.set()
                    elif __debug__:
                        print "unhandled wParam: %i" % wParam
                elif __debug__:
                    print "wrong vid pid: %s" % dbd.dbcc_name
            elif __debug__:
                print "devicetype not right: %i" % dbh.dbch_devicetype
        elif __debug__:
            print "no lParam"

        return True
    
    def onPowerChange(self, wParam, lParam):
        #According to docs we have 20 seconds to process stuff
        if __debug__:
            log.debug("WM_POWERBROADCAST [WPARAM:%i][LPARAM:%i]"%(wParam,lParam))
        if (wParam == Pbt.PBT_APMQUERYSTANDBY or wParam == Pbt.PBT_APMQUERYSUSPEND):
            log.debug("System wants to go into suspend or standby state, closing serial connection")
            self.pyserial_close_event.set()


class SNAPconnectApp(wx.App):
    def _displayMsg(self, msg):
        wx.MessageBox(msg)


def Main(gw, ver):
    gw.app = SNAPconnectApp(False)
    gw.app.frame = SNAPconnectFrame(None, gw.pyserial_close_event, gw.closeEvent, gw.task_queue, gw.done_queue, ver, gw.statusEnabled, gw.portNum)
    gw.app.MainLoop()


if __name__ == '__main__':
    import threading
    import SnapXmlRpc

    srv = SnapXmlRpc.Main()

    if isinstance(srv, SnapXmlRpc.SnapXmlRpcServer):
        srv.guiThread = threading.Thread(target=Main, args=(srv, SnapXmlRpc.VERSION))
        srv.guiThread.setDaemon(True)
        srv.guiThread.start()

        srv.loop()
    else:
        app=wx.App()
        wx.MessageBox(srv.message, "License Error", style=wx.ICON_ERROR|wx.OK)
