# (c) Copyright 2010, Synapse Wireless, Inc.


import binascii

import snap

from snaplib import ScriptsManager
from snaplib import SnappyUploader


def _test_rpc_sent(seq, success):
    """An example HOOK_RPC_SENT function"""
    print "RPC #%i sent: %s" % (seq, success)

def _test_stdin(data):
    """An example HOOK_STDIN function"""
    print "STDIN: %s" % (data)

def _test_traceroute(addr, routes):
    pass

def _test_upload_hook(snappy_upload_obj, result):
    if result == SnappyUploader.SNAPPY_PROGRESS_COMPLETE:
        print "Upload completed successfully to %s" % (binascii.hexlify(snappy_upload_obj.netAddr))
    else:
        print "Upload FAILED to %s" % (binascii.hexlify(snappy_upload_obj.netAddr))

def client_auth(realm):
    """
    An example client authorization function

    Returns the username "public" and password "public" to the remote server
    to use as it's authorization

    realm : The realm specified by the remote server
    """
    return ("public", "public")

def server_auth(realm, username):
    """
    An example sever authentication function

    Returns the password for the specified username in the realm

    realm : This server's realm
    username : The username specified by the remote server
    """
    if username == "public":
        return "public"


class Tester(object):
    def __init__(self):
        self.com = snap.SNAPcom(funcs={'loopback': self.loopback,
                                       'tellVmStat': self.tellVmStat,
                                      },
                                license_file='..\\TestLicense.dat'
                               )
        #self.com.accept_tcp(server_auth, multicast=False)
        #self.com.scheduler.schedule(4, self.com.rpc, "\x07\x07\x07", "loopback", 5)
        #self.com.open_serial(snap.SERIAL_TYPE_RS232, '/dev/ttyUSB0')
        #self.com.open_serial(snap.SERIAL_TYPE_SNAPSTICK100, 0)
        self.com.open_serial(snap.SERIAL_TYPE_RS232, 0)
        #self.com.set_hook(snap.hooks.HOOK_RPC_SENT, _test_rpc_sent)
        #self.com.set_hook(snap.hooks.HOOK_STDIN, _test_stdin)
        #self.com.save_nv_param(snap.NV_AES128_ENABLE_ID, snap.ENCRYPTION_TYPE_PME)
        #self.com.save_nv_param(snap.NV_AES128_ENABLE_ID, snap.ENCRYPTION_TYPE_NONE)
        #self.com.save_nv_param(snap.NV_AES128_KEY_ID, "Synapse!Synapse!")
        self.com.save_nv_param(snap.NV_MESH_ROUTE_AGE_MAX_TIMEOUT_ID, 0)
        self.com.save_nv_param(snap.NV_LOCKDOWN_FLAGS_ID, 0x2)
        #self.com.scheduler.schedule(1, self.com.close_serial, 1, 0)
        #self.com.scheduler.schedule(2, self.com.open_serial, 1, 0)
        #self.com.scheduler.schedule(10, self.com.stop_accepting_tcp)
        #self.com.scheduler.schedule(10, self.com.snaptcp_server.close_all)
        #self.com.data_mode("\x00\x18\x42", "test")
        #self.com.mcast_rpc(1,3,'setYellowLED', 'Test242_1', 1)
        #self.com.rpc("\x00\x81\xb1", "vmStat", 5, 0)
        #self.com.traceroute("\x00\x6e\x9c")
        #self.com.set_hook(snap.hooks.HOOK_TRACEROUTE, _test_traceroute)
        #self.com.mcast_rpc(1,3,'setYellowLED', 'Test242_1', 1)

        # Uploading a SPY file example
        self.com.add_rpc_func('su_recvd_reboot', lambda arg: self.com.spy_upload_mgr.on_recvd_reboot(self.com.rpc_source_addr()))
        f = None
        uploadImage = None
        try:
            f = file('c:\\Synapse_LED_Bridge.spy', 'rb')
            uploadImage = ScriptsManager.getSnappyStringFromExport(f.read())
        except:
            raise IOError("Unable to read the selected SNAPpy image")
        finally:
            if f:
                f.close()
        if uploadImage:
            upload = self.com.spy_upload_mgr.startUpload("\x03\xf8\x62", uploadImage)
            upload.registerFinishedCallback(self.blah)

    def blah(self, res, a):
        f = None
        uploadImage = None
        try:
            f = file('c:\\Synapse_LED_Bridge.spy', 'rb')
            uploadImage = ScriptsManager.getSnappyStringFromExport(f.read())
        except:
            raise IOError("Unable to read the selected SNAPpy image")
        finally:
            if f:
                f.close()
        if uploadImage:
            upload = self.com.spy_upload_mgr.startUpload("\x03\xf8\x62", uploadImage)
            upload.registerFinishedCallback(_test_upload_hook)

    def loop(self):
        poll = self.com.poll
        while True:
            poll()
        self.com.stop_accepting_tcp()

    def loopback(self, arg):
        self.com.rpc(self.com.rpc_source_addr(), "rtnVal", arg)

    def rtnVal(self, val):
        pass

    def tellVmStat(self, arg, val):
        self.com.spy_upload_mgr.onTellVmStat(self.com.rpc_source_addr(), arg, val)


if __name__ == '__main__':
    import logging

    logging.basicConfig(level=logging.DEBUG, format='%(asctime)s:%(msecs)03d %(levelname)-8s %(name)-8s %(message)s', datefmt='%H:%M:%S')

    Tester().loop()
