# (c) Copyright 2012 Synapse Wireless, Inc.
''' The job of this class is to format (snaplayout) a memory image of the passed in Intel HEX file
    that matches the needs of the ATmega128RFA1 "Boot Helper".'''
#
# This was originally a much larger class, but the common functionality was factored out
# into a new class, CoreLayout, which this class now customizes.
#

import CoreLayout
import pyintelhex # pyintelhex.IntelHexReader() will be our "CoreReader"

class ATmega128RFA1_Layout(CoreLayout.CoreLayout):
    CORE_PART1_START_ADDRESS = 0x00000 # Interrupt vectors, main2(), and FIRST PORTION of SWITCH segment
    CORE_PART1_END_ADDRESS   = 0x001FF # <- Expanded AES-128 key starts at 200H

    CORE_PART2_START_ADDRESS = 0x00C00 # REMAINDER of SWITCH segment
    CORE_PART2_END_ADDRESS   = 0x00EFF # <- Script storage starts at F00H

    CORE_PART3_START_ADDRESS = 0x10000 # Bulk of the core goes here in the upper 64K bank
    CORE_PART3_END_ADDRESS   = 0x1F4FF # <- the "Boot Helper" starts at 1F500H

    # The following is used to detect foolish users trying to load OLD firmware (that predates FOTA)
    # We had to change the memory map to support this feature, the older (bad) files have CODE
    # in the portion of FLASH where now only SCRIPTS are allowed.
    CORE_DISALLOWED_START_ADDRESS = 0x00F00 # Only scripts go here in the new (>= 2.4.32) firmware
    CORE_DISALLOWED_END_ADDRESS   = 0x0FFFF # Constants and switch() statement lookup tables did before

    # Non-AES-128 builds are small enough that they don't "spill over" into the now-restricted region (above)
    # Here we are using the FORMER location of the signature (Customer ID+check code) as an additional discriminator
    # (0x00300-0x003FF is blank in the new code)
    OLD_SIGNATURE_START_ADDRESS = 0x00300
    OLD_SIGNATURE_END_ADDRESS   = 0x00303

    def __init__(self, contents):
        super(ATmega128RFA1_Layout, self).__init__(
            pyintelhex.IntelHexReader(),
            contents,
            ((self.CORE_PART1_START_ADDRESS, self.CORE_PART1_END_ADDRESS),
             (self.CORE_PART2_START_ADDRESS, self.CORE_PART2_END_ADDRESS),
             (self.CORE_PART3_START_ADDRESS, self.CORE_PART3_END_ADDRESS)),
            ((self.CORE_DISALLOWED_START_ADDRESS, self.CORE_DISALLOWED_END_ADDRESS),
             (self.OLD_SIGNATURE_START_ADDRESS, self.OLD_SIGNATURE_END_ADDRESS)),
            32 * 1024 # <- because the actual Boot Helper enforces this requirement...
        )

    # for the rest of the story, see CoreLayout.py
