# (c) Copyright 2008-2011, Synapse Wireless Inc.


import logging
log = logging.getLogger("snaplib.%s" % (__name__))


class Registrar(object):
    def __init__(self, events=[]):
        self._registeredEvents = {} #Will hold callbacks for each events
        self._registeredEvents['_callbackRegistered'] = []
        self._registeredEvents['_callbackUnregistered'] = []
        try:
            for event in events:
                assert event not in self._registeredEvents
                self._registeredEvents[event] = []
        except TypeError:
            if __debug__:
                raise

    def _add_event_to_registrar(self, event_name):
        assert event_name not in self._registeredEvents
        try:
            self._registeredEvents[event_name] = []
        except TypeError:
            if __debug__:
                raise

    def _notify_callbacks(self, event, *args, **kwargs):
        """
        Notify all callbacks that the specified event has occurred.
        Note the callbacks are called immediately

        event -- The event that just occurred
        """
        assert event in self._registeredEvents
        
        for callback in self._registeredEvents[event]:
            try:
                callback(*args, **kwargs)
            except (KeyboardInterrupt, SystemExit):
                raise
            except:
                log.exception("An exception occurred while notifying a %s callback:" % (event))
                if __debug__:
                    raise

    _notifyCallbacks = _notify_callbacks # Older non-PEP8 name

    def register_callback(self, event, callback):
        """
        Register a callable object for event notifications

        event -- The event to register for notifications
        callback -- The callable object to register
        """
        assert event in self._registeredEvents
        assert callable(callback)

        if callable(callback):
            if callback not in self._registeredEvents[event]:
                self._registeredEvents[event].append(callback)
                self._notifyCallbacks('_callbackRegistered', event, callback)
                return True
            else:
                log.debug("Callback already exists for %s events" % (event,))
        return False

    registerCallback = register_callback # Older non-PEP8 name

    def unregister_callback(self, event, callback):
        """
        Unregister a callable object of event notifications

        event -- The event to unregister for notifications
        callback -- The callable object to unregister
        """
        assert event in self._registeredEvents
        assert callable(callback)

        try:
            self._registeredEvents[event].remove(callback)
            self._notifyCallbacks('_callbackUnregistered', event, callback)
            return True
        except ValueError:
            pass #If it's not in there than it's removed!
        return False

    unregisterCallback = unregister_callback # Older non-PEP8 name
