from SnapConstants import *

class Vendor(object):
    def __init__(self,id,name):
        self._id = id
        self._name = name

    def getID(self):
        return self._id

    def getName(self):
        return self._name

class CPU(object):
    def __init__(self, id, name, littleEndian, alignEven, maxGlobals, maxCallDepth,supportsOTA):
        self._id = id
        self._name = name
        self._littleEndian = littleEndian
        self._alignEven = alignEven
        self._maxGlobals = maxGlobals
        self._maxCallDepth = maxCallDepth
        self._supportsOTA = supportsOTA

    def getID(self):
        return self._id

    def getName(self):
        return self._name

    def getLocals(self):
        return 64

    def getAlignEven(self):
        return self._alignEven

    def getLittleEndian(self):
        return self._littleEndian

    def getHasTypecodeModifiers(self,version):
        if version>=(2,2,0):
            return True
        return False

    def getHasExtendedTimerHooks(self,version):
        if version>=(2,2,0):
            return True
        return False

    def getHasPower(self,version):
        if version>=(2,2,12):
            return True
        return False

    def getMaxGlobals(self,version):
        return self._maxGlobals

    def getMaxCallDepth(self,version):
        if isinstance(self._maxCallDepth, bool):
            return self._maxCallDepth
        return self._maxCallDepth(version)

    def getSupportsOTAFirmwareUpdate(self,version):
        if isinstance(self._supportsOTA, bool) :
            return self._supportsOTA
        return self._supportsOTA(version)

# Some ATMEGA firmware versions allow OTA updates and others do not
def atmegaSuportsOTA(version):
    if version>=(2,4,32):
        return True
    return False

def atmegaCallDepth(version):
    if version>=(2,4,25):
        return 16
    return 8

class Platform(object):
    def __init__(self, name, internal, id, cpu, vendor, blobSplit):
        self._name = name
        self._internal = internal
        self._id = id
        self._cpu = cpu
        self._vendor = vendor
        self._blobSplit = blobSplit

    def getName(self):
        return self._name

    def isInternal(self):
        return self._internal

    def getID(self):
        return self._id

    def getCPU(self):
        return self._cpu

    def getVendor(self):
        return self._vendor

    def getBlobSplit(self,version):
        if isinstance(self._blobSplit, bool) :
            return self._blobSplit
        return self._blobSplit(version)

def celBlobSplit(version):
    if version>=(2,4,11):
        return True
    return False

_vendors = [
  #          uid           name
     Vendor(SYNAPSE,       "SYNAPSE"),
     Vendor(PANASONIC,     "PANASONIC"),
     Vendor(FREESCALE,     "FREESCALE"),
     Vendor(CEL,           "CEL"),
     Vendor(ATMEL,         "ATMEL"),
     Vendor(SILICON_LABS,  "SILICON LABS"),
     Vendor(SIMULATION,    "SIMULATION"),
     Vendor(PC,            "PC"),
     Vendor(YITRAN,        "YITRAN")
     ]

def getVendorByID(id):
    for v in _vendors:
        if v.getID() == id:
            return v
    return None

_cpus = [
  # uid       id             name             endian align  globals  callDepth          otaUpgrade
      CPU(CPU_MC9S08GT60A,   "MC9S08GT60A",   False, False,   64,       8,                False),
      CPU(CPU_8051,          "8051",          False, False,   64,       8,                False),
      CPU(CPU_ATMEGA128RFA1, "ATMEGA128RFA1", True,  True,   128,       atmegaCallDepth,  atmegaSuportsOTA), # based on current firmware version
      CPU(CPU_SI1000,        "Si1000",        False, False,   64,       8,                False),
      CPU(CPU_ARM7,          "ARM7",          True,  True,   128,       8,                True),
      CPU(CPU_ARM_CORTEX_M3, "CORTEX-M3",     True,  True,   128,       8,                True),
      CPU(CPU_X86,           "x86",           True,  False,   64,       8,                False),
      CPU(CPU_ATMEGA128RFR2, "ATMEGA128RFR2", True,  True,   128,       atmegaCallDepth,  atmegaSuportsOTA), # based on current firmware version
      CPU(CPU_ATMEGA1284RFR2,"ATMEGA1284RFR2",True,  True,   128,       atmegaCallDepth,  atmegaSuportsOTA), # based on current firmware version
     #
     #CPU_MC9S08QE      : CPU("MC9S08QE",    None,  None,     0,       0,                None),
     #CPU_COLDFIRE      : CPU("COLDFIRE",    None,  None,     0,       0,                None),
     #CPU_SPARC_LEON    : CPU("SPARC_LEON",  None,  None,     0,       0,                None),
     ]

def getCPUByID(id):
    for c in _cpus:
        if c.getID() == id:
            return c
    return None

_platforms = [
  #             name         internal  platID              CPU                       Vendor                  blobSplit
     Platform('RFEngine',     False,   RF_ENGINE,    getCPUByID(CPU_MC9S08GT60A),    getVendorByID(SYNAPSE),  False),
     Platform('RF100',        False,   RF_ENGINE,    getCPUByID(CPU_MC9S08GT60A),    getVendorByID(SYNAPSE),  False),
     Platform('ZIC2410_SE',   False,   CEL_ZIC2410,  getCPUByID(CPU_8051),           None,                    celBlobSplit),
     Platform('PAN4555_SE',   False,   MC1321X,      getCPUByID(CPU_MC9S08GT60A),    None,                    False),
     Platform('PAN4561_SE',   False,   MC1321X,      getCPUByID(CPU_MC9S08GT60A),    None,                    False),
     Platform('RF300',        False,   RF300,        getCPUByID(CPU_SI1000),         None,                    False),
     Platform('SM300',        False,   SM300,        getCPUByID(CPU_SI1000),         None,                    False),
     Platform('SM301',        False,   SM301,        getCPUByID(CPU_SI1000),         None,                    False),
     Platform('RF301',        False,   SM301,        getCPUByID(CPU_SI1000),         None,                    False), # because the "RF301" is really a SM301 mounted on a carrier board!
     Platform('SM700',        False,   MC1322X,      getCPUByID(CPU_ARM7),           None,                    False),
     Platform('RF200',        False,   RF200_PFM,    getCPUByID(CPU_ATMEGA128RFA1),  None,                    False),
     Platform('RF266',        False,   RF266_PFM,    getCPUByID(CPU_ATMEGA128RFA1),  None,                    False),
     Platform('SS200',        False,   RF200_PFM,    getCPUByID(CPU_ATMEGA128RFA1),  None,                    False), # because we just copy the RF200 firmware and give it a SS200 filename
     Platform('SM200',        False,   SM200_PFM,    getCPUByID(CPU_ATMEGA128RFA1),  None,                    False),
     Platform('STM32W108xB',  False,   STM32W108xB,  getCPUByID(CPU_ARM_CORTEX_M3),  None,                    False),
     Platform('Simulated',    True,    SIMULATED,    getCPUByID(CPU_X86),            None,                    False),
     #
     Platform('SM222',           True,   SM222_PFM,          getCPUByID(CPU_ATMEGA128RFR2),  None, False),
     Platform('ATmega1284RFR2',  False,  ATmega1284RFR2_PFM, getCPUByID(CPU_ATMEGA1284RFR2), None, False),
     Platform('SM220',           False,  SM220UF1_PFM,       getCPUByID(CPU_ATMEGA128RFA1),  None, False),
     Platform('ATmega128RFR2',   False,  ATmega128RFR2_PFM,  getCPUByID(CPU_ATMEGA128RFR2),  None, False),

     #
     #RF400
     #PAN4555
     #FLEXIS_EVB
     #MC1321x_EVK
     #ATmega128RFA1
     #SNAPCOM
     #SI100X_DSSS
     #IT700
     #SI100X_KADEX, SI100X_FHSS
     #CEL_ZIC2411
     #RN_G2C547
     ]

def getPlatform(name):
    return _platforms.get(name)

def getPlatformNames():
    ret = []
    for plat in _platforms:
        ret.append(plat)
    ret.sort()
    return ret

def getCPUDescriptions():
    ret = []
    for cpu in _cpus.values():
        ret.append(cpu.getName())
    ret.sort()
    return ret


if __name__ == '__main__':

    print 'here'


