# -*- coding: utf-8 -*-

# (c) Copyright 2007-2012 Synapse Wireless, Inc.
"""Helper fuctions for SnapConstants"""

import pprint

#Yea, I agree we shouldn't do import * but this module is designed to be a wrapper for everything in SnapConstants
from SnapConstants import *


MIN_TRACEROUTE_VER = '2.2.6'


def getErrorMessage(id):
  try:
    return ERROR_MESSAGES[id]
  except KeyError:
    return "Unknown"

def getNvParamName(id):
  try:
    return NV_PARAM_STRINGS[id]
  except KeyError:
    return "Unknown"

def NvIntWrapper(value, output=True):
  #Note this will display ints as unsigned
  if output:
    if value is not None:
      return str(value % 2**16)
    return str(value)
  else:
    if value == 'None':
      return 0
    else:
      return int(value)

def NvStrWrapper(value, output=True):
  #Wrapping the str fuction since we are expecting the output param
  if output:
    if value is not None:
      return str(value)
    else:
      return ''
  else:
    if value:
      return str(value)
    else:
      return None

def NvStringLiteralConverter(value, output=True):
  """Converts text into python string"""
  if output:
    if value is None:
      rtnVal = ''
    else:
      rtnVal = pprint.pformat(value)[1:-1] # Remove quotes
  else:
    rtnVal = eval("'%s'" % (value))

  return rtnVal

def NvMacConverter(value, output=True):
  import binascii

  if output:
    return ("%s:%s:%s:%s:%s:%s:%s:%s" % (binascii.hexlify(value[0]), binascii.hexlify(value[1]), binascii.hexlify(value[2]),
                                         binascii.hexlify(value[3]), binascii.hexlify(value[4]), binascii.hexlify(value[5]),
                                         binascii.hexlify(value[6]), binascii.hexlify(value[7]))).upper()
  else:
    return binascii.unhexlify(value.replace(':', ''))

def NvHexConverter(value, output=True):
  if output:
    if value:
      try:
        return "0x%04X" % (value % 2**16)
      except TypeError:
        return ''
    else:
      return 'None'
  else:
    if len(value) > 2 and value[:2].lower() == "0x":
      return int(value, 16)
    else:
      return int(value)

def getNvParamType(id):
  try:
    return NV_PARAM_TYPES[id]
  except KeyError:
    return NvIntWrapper

def isNvParamEditable(id):
  return (id not in UNEDITABLE_NVPARAMS)

def NvBoolWrapper(value, output=True):
  if output:
    return str(value)
  else:
    if value == str(True):
      return True
    return False


NV_PARAM_TYPES = {NV_MAC_ADDR_ID: NvMacConverter,
                 NV_NETWORK_ID: NvHexConverter,
                 NV_CHANNEL_ID: NvIntWrapper,
                 NV_GROUP_INTEREST_MASK_ID: NvIntWrapper,
                 NV_GROUP_FORWARDING_MASK_ID: NvIntWrapper,
                 NV_MFG_DATE_ID: NvStrWrapper,
                 NV_DEVICE_NAME_ID: NvStrWrapper,
                 NV_SYSTEM_ERROR_ID: NvIntWrapper,
                 NV_DEVICE_NAME_ID: NvStrWrapper,
                 NV_DEVICE_TYPE_ID: NvStrWrapper,
                 NV_FEATURE_BITS_ID: NvHexConverter,
                 NV_DEFAULT_UART_ID: NvIntWrapper,
                 NV_UART_DM_TIMEOUT_ID: NvIntWrapper,
                 NV_UART_DM_THRESHOLD_ID: NvIntWrapper,
                 NV_UART_DM_INTERCHAR_ID: NvIntWrapper,
                 NV_CARRIER_SENSE_ID: NvBoolWrapper,
                 NV_COLLISION_DETECT_ID: NvBoolWrapper,
                 NV_COLLISION_AVOIDANCE_ID: NvBoolWrapper,
                 NV_SNAP_MAX_RETRIES_ID: NvIntWrapper,
                 NV_MESH_ROUTE_AGE_MAX_TIMEOUT_ID: NvIntWrapper,
                 NV_MESH_ROUTE_AGE_MIN_TIMEOUT_ID: NvIntWrapper,
                 NV_MESH_ROUTE_NEW_TIMEOUT_ID: NvIntWrapper,
                 NV_MESH_ROUTE_USED_TIMEOUT_ID: NvIntWrapper,
                 NV_MESH_ROUTE_DELETE_TIMEOUT_ID: NvIntWrapper,
                 NV_MESH_RREQ_TRIES_ID: NvIntWrapper,
                 NV_MESH_RREQ_WAIT_TIME_ID: NvIntWrapper,
                 NV_MESH_INITIAL_HOPLIMIT_ID: NvIntWrapper,
                 NV_MESH_MAX_HOPLIMIT_ID: NvIntWrapper,
                 NV_MESH_SEQUENCE_NUMBER_ID: NvIntWrapper,
                 NV_MESH_OVERRIDE_ID: NvIntWrapper,
                 NV_MESH_PENALTY_LQ_THRESHOLD_ID: NvIntWrapper,
                 NV_MESH_REJECTION_LQ_THRESHOLD_ID: NvIntWrapper,
                 NV_SNAP_LQ_THRESHOLD_ID: NvIntWrapper,
                 NV_SNAPPY_CRC_ID: NvIntWrapper,
                 NV_SYS_PLATFORM_ID: NvStrWrapper,
                 NV_AES128_ENABLE_ID: NvIntWrapper,
                 NV_AES128_KEY_ID: NvStringLiteralConverter,
                 NV_LOCKDOWN_FLAGS_ID: NvIntWrapper,
                 NV_ALT_RADIO_TRIM_ID: NvHexConverter,
                 NV_SNAPPY_CRC_ID: NvHexConverter,
                 NV_VENDOR_SETTINGS_ID: NvHexConverter,
                 NV_CS_CD_LEVEL_ID: NvIntWrapper}

NV_PARAM_HELP_STRINGS = {NV_MAC_ADDR_ID: "The MAC address for this device (NV %d)" % (NV_MAC_ADDR_ID,),
                         NV_NETWORK_ID: "The SNAP network ID this device should start on (NV %d)" % (NV_NETWORK_ID,),
                         NV_CHANNEL_ID: "The SNAP channel this device should start on (NV %d)" % (NV_CHANNEL_ID,),
                         NV_SNAP_MAX_RETRIES_ID: "The maximum number of retries for a unicast message (NV %d)" % (NV_SNAP_MAX_RETRIES_ID,),
                         NV_GROUP_INTEREST_MASK_ID: "A mask defining what multicast groups this device is interested in (NV %d)" % (NV_GROUP_INTEREST_MASK_ID,),
                         NV_GROUP_FORWARDING_MASK_ID: "A mask defining which multicast groups this device will forward (TTL permitting) (NV %d)" % (NV_GROUP_FORWARDING_MASK_ID,),
                         NV_MFG_DATE_ID: "The manufacturing date of this device (NV %d)" % (NV_MFG_DATE_ID,),
                         NV_DEVICE_NAME_ID: "The user defined name of this device (NV %d)" % (NV_DEVICE_NAME_ID,),
                         NV_DEVICE_TYPE_ID: "The user programmed device type (NV %d)" % (NV_DEVICE_TYPE_ID,),
                         NV_FEATURE_BITS_ID: "A mask defining what features are enabled on this device (NV %d)\n0b0000,0000,0001 = Enable UART 0\n0b0000,0000,0010 = Enable hardware flow control on UART 0\n0b0000,0000,0100 = Enable UART 1\n0b0000,0000,1000 = Enable hardware flow control on UART 1\n0b0000,0001,0000 = Enable hardware power amplifier\n0b0000,0010,0000 = Enable external power-down output\n0b0000,0100,0000 = Enable alternate clock source\n0b0000,1000,0000 = Enable DS_AUDIO\n0b0001,0000,0000 = Enable second data CRC\n0b0010,0000,0000 = Reduce Tx power levels to world-wide settings" % (NV_FEATURE_BITS_ID,),
                         NV_DEFAULT_UART_ID: "The default UART this device should connect packet serial to (NV %d)" % (NV_DEFAULT_UART_ID,),
                         NV_UART_DM_TIMEOUT_ID: "How many milliseconds MAX until data is pushed to the other end (NV %d)" % (NV_UART_DM_TIMEOUT_ID,),
                         NV_UART_DM_THRESHOLD_ID: "How big to let the packet grow (including header) before pushing to the other end (NV %d)" % (NV_UART_DM_THRESHOLD_ID,),
                         NV_UART_DM_INTERCHAR_ID: "How many milliseconds between characters before pushing to the other end (NV %d)" % (NV_UART_DM_INTERCHAR_ID,),
                         NV_CARRIER_SENSE_ID: "Listen to see if someone is transmitting before this node tries to transmit (NV %d)" % (NV_CARRIER_SENSE_ID,),
                         NV_COLLISION_DETECT_ID: "Listen to see if someone is transmitting after you send a multicast (NV %d)" % (NV_COLLISION_DETECT_ID,),
                         NV_COLLISION_AVOIDANCE_ID: "Use an initial random backoff time to avoid collisions (NV %d)" % (NV_COLLISION_AVOIDANCE_ID,),
                         NV_MESH_ROUTE_AGE_MAX_TIMEOUT_ID: "The maximum age in milliseconds a route is valid (NV %d)" % (NV_MESH_ROUTE_AGE_MAX_TIMEOUT_ID,),
                         NV_MESH_ROUTE_AGE_MIN_TIMEOUT_ID: "The minimum age in milliseconds a route is valid (NV %d)" % (NV_MESH_ROUTE_AGE_MIN_TIMEOUT_ID,),
                         NV_MESH_ROUTE_NEW_TIMEOUT_ID: "The timeout period in milliseconds for a new but unused route (NV %d)" % (NV_MESH_ROUTE_NEW_TIMEOUT_ID,),
                         NV_MESH_ROUTE_USED_TIMEOUT_ID: "The timeout period in milliseconds for a route that has been used (NV %d)" % (NV_MESH_ROUTE_USED_TIMEOUT_ID,),
                         NV_MESH_ROUTE_DELETE_TIMEOUT_ID: "The timeout period in milliseconds for a route that has been deleted (NV %d)" % (NV_MESH_ROUTE_DELETE_TIMEOUT_ID,),
                         NV_MESH_RREQ_TRIES_ID: "The number of times to retry sending a route request (NV %d)" % (NV_MESH_RREQ_TRIES_ID,),
                         NV_MESH_RREQ_WAIT_TIME_ID: "The time in milliseconds to wait for a route reply per request (NV %d)" % (NV_MESH_RREQ_WAIT_TIME_ID,),
                         NV_MESH_INITIAL_HOPLIMIT_ID: "The initial hop count to send a route request with (NV %d)" % (NV_MESH_INITIAL_HOPLIMIT_ID,),
                         NV_MESH_MAX_HOPLIMIT_ID: "The maximum number of hops to send a route request with (NV %d)" % (NV_MESH_MAX_HOPLIMIT_ID,),
                         NV_MESH_SEQUENCE_NUMBER_ID: "The starting mesh sequence number for mesh routing packets (NV %d)" % (NV_MESH_SEQUENCE_NUMBER_ID,),
                         NV_MESH_OVERRIDE_ID: "Override mesh - use mesh routing but do not route for other nodes (NV %d)" % (NV_MESH_OVERRIDE_ID,),
                         NV_MESH_PENALTY_LQ_THRESHOLD_ID: "Link Quality (in dBm) at which to penalize a route by one hop (NV %d)" % (NV_MESH_PENALTY_LQ_THRESHOLD_ID,),
                         NV_MESH_REJECTION_LQ_THRESHOLD_ID: "Link Quality (in dBm) at which to ignore a route completely (NV %d)" % (NV_MESH_REJECTION_LQ_THRESHOLD_ID,),
                         NV_SNAP_LQ_THRESHOLD_ID: "Link Quality (in dBm) at which to ignore a node completely (NV %d)" % (NV_SNAP_LQ_THRESHOLD_ID,),
                         NV_SNAPPY_CRC_ID: "The CRC of the current SNAPpy script generated at startup (NV %d)" % (NV_SNAPPY_CRC_ID,),
                         NV_SYS_PLATFORM_ID: "Hardware platform name => snapsys.platform in SNAPpy scripts (NV %d)" % (NV_SYS_PLATFORM_ID,),
                         NV_AES128_ENABLE_ID: "Specify encryption type (NV %d)" % (NV_AES128_ENABLE_ID,),
                         NV_AES128_KEY_ID: "The encryption key - must be exactly 16 characters in length (NV %d)" % (NV_AES128_KEY_ID,),
                         NV_LOCKDOWN_FLAGS_ID: "The bitmask to enable/disable node lockdowns (NV %d)" % (NV_LOCKDOWN_FLAGS_ID,),
                         NV_ALT_RADIO_TRIM_ID: "The trim value for the radio transceiver's crystal oscillator (NV %d)" % (NV_ALT_RADIO_TRIM_ID,),
                         NV_VENDOR_SETTINGS_ID: "A mask defining what vendor features are enabled on this device (NV %d)" % (NV_VENDOR_SETTINGS_ID,),
                         NV_CS_CD_LEVEL_ID: "Link Quality (in dBm) to be used as the Carrier Sense and Collision Detect threshold (NV %d)" % (NV_CS_CD_LEVEL_ID,)}

PID_STRINGS = ['Reserved', 'Data Mode Broadcast', 'Data Mode PTP', 'NACK', 'RPC', 'ACK', 'Mesh Broadcast', 'Mesh PTP', 'Multicast RPC', 'Trace Route']
UNEDITABLE_NVPARAMS = (NV_MAC_ADDR_ID, NV_ALT_RADIO_TRIM_ID, NV_VENDOR_SETTINGS_ID)

#From Knobs.h
SNAP_RPC_ACK_TIMEOUT = 20 # ms to wait for ACK before resending
SNAP_RPC_MAX_RETRIES = 8 # number of times to resend before aborting tx
SNAP_DEFAULT_MAX_HOPLIMIT = 5

#We use this calculation in a few places, but can now be user changed (this is the default)
SNAP_RPC_TIMEOUT = SNAP_RPC_ACK_TIMEOUT*SNAP_RPC_MAX_RETRIES*SNAP_DEFAULT_MAX_HOPLIMIT

TYPE_CODE_STRING = {TC_NONE: 'None',
                    TC_INT: 'Int',
                    TC_STRING: 'String',
                    TC_FUNC: 'Func',
                    TC_CFUNC: 'CFunc',
                    TC_BOOL: 'Bool',
                    TC_TUPLE: 'Tuple',
                    TC_UNBOUND: 'Unbound',
                    }

def getPidString(pid):
  rtn = 'Unknown PID'
  try:
    rtn = PID_STRINGS[pid]
  except IndexError:
    try:
      rtn = PID_STRINGS[pid-PID_ENCRYPTION_MASK] + '*'
    except IndexError:
      pass
  except TypeError:
    pass

  return rtn


SNAP_SNYF_LEN_OFS = 0
SNAP_SNYF_PID_OFS = 1

TYPE_CODE_STRING = {TC_NONE: 'None',
                    TC_INT: 'Int',
                    TC_STRING: 'String',
                    TC_FUNC: 'Func',
                    TC_CFUNC: 'CFunc',
                    TC_BOOL: 'Bool',
                    TC_TUPLE: 'Tuple',
                    TC_UNBOUND: 'Unbound',
                    }

def get_arg_type_string(type_code):
  return TYPE_CODE_STRING.get(type_code, 'Unknown')


cpu_descriptions = {CPU_MC9S08GT60A    : 'MC9S08GT60A',
                    CPU_8051           : '8051',
                    CPU_ATMEGA128RFA1  : 'ATMEGA128RFA1',
                    CPU_SI1000         : 'Si1000',
                    CPU_ARM7           : 'ARM7',
                    CPU_ARM_CORTEX_M3  : 'CORTEX-M3',
                    CPU_X86            : 'x86',
                    CPU_ATMEGA128RFR2  : 'ATMEGA128RFR2',
                    CPU_ATMEGA1284RFR2 : 'ATMEGA1284RFR2',}

cpu_descriptions_to_id = {'MC9S08GT60A'   : CPU_MC9S08GT60A,
                          '8051'          : CPU_8051,
                          'ATMEGA128RFA1' : CPU_ATMEGA128RFA1,
                          'Si1000'        : CPU_SI1000,
                          'ARM7'          : CPU_ARM7,
                          'CORTEX-M3'     : CPU_ARM_CORTEX_M3,
                          'x86'           : CPU_X86,
                          'ATMEGA128RFR2' : CPU_ATMEGA128RFR2,
                          'ATMEGA1284RFR2': CPU_ATMEGA1284RFR2,}

# Make sure this has (at least) one entry for every item in SnapConstants.py::platforms[] (except 'Portal')
platforms_strings_to_ids = {'RFEngine'    : RF_ENGINE,
                            'RF100'       : RF_ENGINE,
                            'ZIC2410_SE'  : CEL_ZIC2410,
                            'PAN4555_SE'  : MC1321X,
                            'PAN4561_SE'  : MC1321X,
                            'RF300'       : RF300,
                            'SM300'       : SM300,
                            'SM301'       : SM301,
                            'RF301'       : SM301, # because the "RF301" is really a SM301 mounted on a carrier board!
                            'SM700'       : MC1322X,
                            'RF200'       : RF200_PFM,
                            'RF266'       : RF266_PFM,
                            'SS200'       : RF200_PFM, # because we just copy the RF200 firmware and give it a SS200 filename...
                            'SM200'       : SM200_PFM,
                            'STM32W108xB' : STM32W108xB,
                            'Simulated'   : SIMULATED,
                            #'RF400'       : RF400_PFM,
                            'SM222'          : SM222_PFM,
                            'SM220'          : SM220UF1_PFM,
                            }

platforms_to_cpus = {'RFEngine'   : CPU_MC9S08GT60A,
                     'PAN4561_SE' : CPU_MC9S08GT60A,
                     'PAN4555_SE' : CPU_MC9S08GT60A,
                     'ZIC2410_SE' : CPU_8051,
                     'RF100'      : CPU_MC9S08GT60A,
                     'RF200'      : CPU_ATMEGA128RFA1,
                     'RF266'      : CPU_ATMEGA128RFA1,
                     'SS200'      : CPU_ATMEGA128RFA1,
                     'SM200'      : CPU_ATMEGA128RFA1,
                     'RF300'      : CPU_SI1000,
                     'SM300'      : CPU_SI1000,
                     'RF301'      : CPU_SI1000,
                     'SM301'      : CPU_SI1000,
                     'SM700'      : CPU_ARM7,
                     'Simulated'  : CPU_X86,
                     #'RF400'      : CPU_ATMEGA128RFA1,
                     'SM222'          : CPU_ATMEGA128RFR2,
                     'SM220'          : CPU_ATMEGA128RFA1,
                     }

ENCRYPTION_TYPE_NONE = 0
ENCRYPTION_TYPE_AES128 = 1
ENCRYPTION_TYPE_BASIC = ENCRYPTION_TYPE_PME = 2

PROTOCOL_SNAP_OVER_TCP = 0
PROTOCOL_SNAP_OVER_XMLRPC = 1
# Maybe PROTOCOL_SNAP_OVER_6LOWPAN someday...
