# (c) Copyright 2010, Synapse Wireless, Inc.
"""This module understand how AES/PME is implemented in the various SNAP packets"""


import SnapConstantsHelper


def _get_data(pkt, crypto, decode=True):
    if not crypto or not crypto.has_valid_key:
        return pkt

    assert len(pkt) > SnapConstantsHelper.UNI_PKT_DATA

    pid = ord(pkt[SnapConstantsHelper.RPC_PID_OFS])
    seq_num = ord(pkt[SnapConstantsHelper.RPC_SEQ_OFS])
    offset = 0

    if decode:
        if not (pid & SnapConstantsHelper.PID_ENCRYPTION_MASK):
            return pkt
        else:
            pid ^= SnapConstantsHelper.PID_ENCRYPTION_MASK

    if pid in (SnapConstantsHelper.PID_DM_BCAST, SnapConstantsHelper.PID_MULTICAST_RPC):
        offset = SnapConstantsHelper.MULTI_PKT_DATA
    elif pid in (SnapConstantsHelper.PID_DM_PTP, SnapConstantsHelper.PID_RPC, SnapConstantsHelper.PID_TRACE_ROUTE):
        offset = SnapConstantsHelper.FWD_UNI_PKT_DATA
    elif pid == SnapConstantsHelper.PID_MESH_BCAST:
        offset = SnapConstantsHelper.MESH_MULTI_PKT_DATA
    elif pid == SnapConstantsHelper.PID_MESH_PTP:
        offset = SnapConstantsHelper.UNI_PKT_DATA

    assert offset
    assert seq_num >= 0

    if decode:
        pkt = pkt[:offset] + crypto.decrypt(seq_num, pkt[offset:])
        return pkt[:ord(pkt[0])+1] # Trim to what the length of the packet is supposed to be
    else:
        return (pkt[:SnapConstantsHelper.RPC_PID_OFS] + 
                "%c" % (pid ^SnapConstantsHelper.PID_ENCRYPTION_MASK) + 
                pkt[SnapConstantsHelper.RPC_PID_OFS+1:offset] + 
                crypto.encrypt(seq_num, pkt[offset:]))

def decode_pkt(pkt, crypto):
    return _get_data(pkt, crypto, decode=True)

def encode_pkt(pkt, crypto):
    return _get_data(pkt, crypto, decode=False)

