# (c) Copyright 2007-2012, Synapse Wireless, Inc.


import sys
import platform
import os
import ctypes

from pycrc import pycrc


crc_lib = None
if platform.machine() == 'armv5tejl':   # the e10, lp400, lp410, etc
    # Example compile steps using gcc:
    # python pycrc.py --model crc-16 --algorithm table-driven --table-idx-width 8 --poly=0x8005 --reflect-out=True --reflect-in=True --xor-in=0 --xor-out=0 --generate c-main -o crc.c
    # python pycrc.py --model crc-16 --algorithm table-driven --table-idx-width 8 --poly=0x8005 --reflect-out=True --reflect-in=True --xor-in=0 --xor-out=0 --generate h -o crc.c
    # arm-linux-gcc -c -fPIC crc.c -o crc_x86.o
    # arm-linux-gcc -shared crc_x86.o -lrt -o snap_crc_armv5tejl.so
    try:
        crc_lib = ctypes.cdll.LoadLibrary(os.path.join(os.path.curdir, 'snap_crc_armv5tejl.so'))
    except:
        pass
elif platform.machine() == 'x86_64':
    # Example compile steps using gcc:
    # python pycrc.py --model crc-16 --algorithm table-driven --table-idx-width 8 --poly=0x8005 --reflect-out=True --reflect-in=True --xor-in=0 --xor-out=0 --generate c-main -o crc.c
    # python pycrc.py --model crc-16 --algorithm table-driven --table-idx-width 8 --poly=0x8005 --reflect-out=True --reflect-in=True --xor-in=0 --xor-out=0 --generate h -o crc.c
    # gcc -c -fPIC crc.c -o crc_x86.o
    # gcc -shared crc_x86.o -lrt -o snap_crc_x86_64.so
    try:
        crc_lib = ctypes.cdll.LoadLibrary(os.path.join(os.path.curdir, 'snap_crc_x86_64.so'))
    except:
        pass
#elif platform.machine() == 'x86': # NOT currently working correctly but isn't high priority
    ## Example compile steps using gcc under cygwin:
    ## python pycrc.py --model crc-16 --algorithm table-driven --table-idx-width 8 --poly=0x8005 --reflect-out=True --reflect-in=True --xor-in=0 --xor-out=0 --generate c-main -o crc.c
    ## python pycrc.py --model crc-16 --algorithm table-driven --table-idx-width 8 --poly=0x8005 --reflect-out=True --reflect-in=True --xor-in=0 --xor-out=0 --generate h -o crc.c
    ## gcc -c -fPIC -mno-cygwin crc.c -o crc_x86.o
    ## gcc -shared -mno-cygwin crc_x86.o -lrt -o snap_crc_x86.so
    #if sys.platform == 'win32':
        #crc_lib = ctypes.windll.LoadLibrary('snap_crc_x86.so')
    #else:
        #crc_lib = cdll.LoadLibrary('snap_crc_x86.so')

if crc_lib is None:
    crc_generator = pycrc.Crc(width=16, poly=0x8005L, reflect_in=True, xor_in=0x0000, reflect_out=True, xor_out=0x0000, table_idx_width=8)
else:
    crc_update = crc_lib.crc_update
    crc_update.restype = ctypes.c_uint16
    crc_update.argtypes = [ctypes.c_uint16, ctypes.c_char_p, ctypes.c_ulong]


def getSnapCrc(data):
    """Using the data passed in, create a CRC16 with the CRC options used for SNAP packets"""
    if crc_lib is None:
        return crc_generator.table_driven(data)
    return crc_update(0, ctypes.c_char_p(data), len(data))

def getCrc(buf):
    opt = pycrc.Options()
    opt.parse('--model crc-16 --algorithm table-driven'.split())
    opt.CheckString = buf

    return pycrc.check_string(opt)


if __name__=='__main__':
    print "0x%x" % getCrc("The quick brown fox jumped over the lazy dog's back!")
    print "0x%x" % getSnapCrc("The quick brown fox jumped over the lazy dog's back!")
