# (c) Copyright 2009, Synapse Wireless, Inc.
"""Various interface definitions for SNAP packets"""


import SnapConstantsHelper
import binascii

class Interface(object):
    """The default interface definition"""
    def __init__(self, intf_type=SnapConstantsHelper.INTF_TYPE_UNKNOWN):
        self.intf_type = intf_type

    def write(self, data):
        """Write the specified data out this interface, return if it was sent or queued"""
        return False

    def __str__(self):
        return "Unknown Interface"


class TcpIpInterface(Interface):
    """A TCP/IP interfaces, includes wifi & ethernet"""
    def __init__(self, ip_addr, port, multicast=False):
        Interface.__init__(self, SnapConstantsHelper.INTF_TYPE_ETH)
        self.ip_addr = ip_addr
        self.port = port
        self.multicast = multicast

    def __str__(self):
        return "TCP/IP Interface (%s:%i)" % (self.ip_addr, self.port)


class Rs232Interface(Interface):
    """An RS-232 interface"""
    def __init__(self, port_num):
        Interface.__init__(self, SnapConstantsHelper.INTF_TYPE_SERIAL)
        self.port_num = port_num

    def __str__(self):
        return "RS-232 Interface: %s" % (self.port_num)


class UsbInterface(Interface):
    """A USB interface"""
    def __init__(self, port_num, type=SnapConstantsHelper.INTF_TYPE_SILABS_USB):
        assert type in [SnapConstantsHelper.INTF_TYPE_SILABS_USB, SnapConstantsHelper.INTF_TYPE_SNAPSTICK]
        Interface.__init__(self, type)
        self.port_num = port_num

    def __str__(self):
        return "USB Interface (%i): %i" % (self.intf_type, self.port_num)

class BtleSerialInterface(Interface):
    """A BTLE Serial interface"""
    def __init__(self, address):
        Interface.__init__(self, SnapConstantsHelper.INTF_TYPE_BTLE_SERIAL)
        if 'peripheral' in address:
            self.address = address[-3:] #Get the address from the end
            self.btle_type = 'peripheral'
        else:
            self.address = address
            self.btle_type = 'central'

    def __str__(self):
        return "BTLE Serial Interface (%s): %s" % (self.btle_type, binascii.hexlify(self.address))
