# (c) Copyright 2010, Synapse Wireless, Inc.
"""
Poor Man's Encryption for SNAP

Requires the key to be exactly 16 bytes, but could care less about the plaintext length
"""


BLOCK_SIZE = 16


class PoorManEncryption(object):
    """Python implementation of the SNAP C version of Poor Man's Encryption"""
    key   = None      # The actual key

    def __init__(self, key, enabled=True):
        self.key = key # Note that self.key is a property, and this statement causes set_key() to get invoked
        self.enabled = enabled

    def decrypt(self, sequenceNumber, cipherText):
        keyIndex = sequenceNumber % len(self._key)
        plainText = ''
        index = 0
        count = len(cipherText)
        while count > 0:
            plainText += chr( ord( cipherText[index] ) ^ ord(self._key[keyIndex]) )
            index += 1
            count -= 1
            keyIndex += 1
            if keyIndex >= BLOCK_SIZE:
                keyIndex = 0

        return plainText

    def encrypt(self, sequenceNumber, plainText):
        # PME is it's own inverse
        cipherText = self.decrypt(sequenceNumber, plainText)
        return cipherText

    def get_has_valid_key(self):
        """Is PME enabled and do we have a valid key"""
        return self.enabled and hasattr(self, '_key') and len(self._key) == BLOCK_SIZE

    def set_key(self, value):
        if value and len(value) == BLOCK_SIZE:
            self._key = value
        else:
            self._key = ""

    key = property(None, set_key) # no getter, setter is set_key
    has_valid_key = property(get_has_valid_key)
