# (c) Copyright 2010, Synapse Wireless, Inc.
"""Keeps track of the system level sequence numbers for SNAP packets"""

import random

_local_seq = {} # {address: sequence_number}

def get_number(local_addr='system'):
    """Return the next available sequence number for the given address

    Args:
        local_addr : The address needing the next sequence number

    Return:
        A sequence number in the range of [0x00, 0xFF]
    """
    global _local_seq
    try:
        if _local_seq[local_addr] >= 255:
            _local_seq[local_addr] = 0
        else:
            _local_seq[local_addr] += 1
    except KeyError:
        # Initial value should be random
        _local_seq[local_addr] = random.randint(0,254)

    return _local_seq[local_addr]