#!/usr/bin/python

SLOW_BLINK = """S00D0000626C696E6B2E7372656307
S21440803C4800A0E3800480E38018A0E3101080E59D
S21440804CE027A0E3082080E5402DA0E30C2080E547
S21440805C24209FE50000A0E1012052E2FCFFFF1A1D
S21440806C001080E5B729A0E36C2E82E30000A0E167
S21440807C012052E2FCFFFF1AF0FFFFEA404B4C0097
S80440803CAF
"""

FAST_BLINK = """S00D0000626C696E6B2E7372656307
S21440803C4800A0E3800480E38018A0E3101080E59D
S21440804CE027A0E3082080E5402DA0E30C2080E547
S21440805CF42AA0E3902F82E30000A0E1012052E234
S21440806CFCFFFF1A001080E5602AA0E36A2D82E32D
S21440807C0000A0E1012052E2FCFFFF1AEFFFFFEAEE
S80440803CAF
"""

COMPRESS = False
CODE     = FAST_BLINK
#CODE     = SLOW_BLINK
#with open("test.srec", "rt") as f:
#    CODE = f.read()

PORT    = 26
RX_CHAR = "\xF9"
TX_CHAR = "\xF6"
TIMEOUT = 1
BAUD    = 115200

from HuffmanPacker import CompressSRec, EncodeSRec
from serial import Serial
import struct

Debug = ""

def Fetch(N):
    global s, Debug
    RetVal, Dotted = "", False
    while len(RetVal) < N:
        A = s.read(N - len(RetVal))
        if A:
            RetVal += A
            Debug += A
        else:
            Dotted = True
            print ".",
    if Dotted:
        print
    return RetVal

if COMPRESS:
    CodeImage = CompressSRec(CODE)
else:
    CodeImage = EncodeSRec(CODE)

s = Serial(PORT, BAUD, timeout=TIMEOUT)

print "Reboot the device..."
while Fetch(1) != RX_CHAR:
    pass

s.write(TX_CHAR)
print "Connected"

# 'e' - Erase entire chip application area. Responds with '\r'.
##print "Erasing chip..."
##s.write('e')
##assert(Fetch(1) == '\r')

# 'b' - Test block support. Responds with 'Y' BlockSizeHi BlockSizeLo.
print "Testing block support"
s.write('b')
assert(Fetch(3) == "Y\x10\x00")

# 'I' - Read bootloader version. Responds with BootloaderVer Block0Start
# Block1Start Block1End.
print "Reading version info"
s.write('I')
Resp = Fetch(3)
assert((Resp == "\x01\xE0\x00") or (Resp == "\x01\xD0\x00"))

# 'A' AddrHi AddrMid AddrLo - Sets the Flash address. Responds with '\r'.
print "Writing Flash..."
s.write('A\x00\x00')
assert(Fetch(1) == "\r")
# 'B' LenHi LenLo [ series of data bytes ] - Program a block of memory starting
# at the address set previously or the end of the last block operation
# (program/read). Responds with ChecksumHi ChecksumLo.
Sum = sum([ord(i) for i in CodeImage])
s.write(struct.pack(">cH%ds" % len(CodeImage), 'B', len(CodeImage),
    CodeImage))
#for i in xrange(0,len(CodeImage),16):
#    for j in xrange(16):
#        print '%02X' % ord(CodeImage[i+j]),
#    print
assert(Fetch(2) == struct.pack(">H", Sum & 0xFFFF))

print "Reading Flash..."
s.write('A\x00\x00')
assert(Fetch(1) == "\r")
# 'g' LenHi LenLo - Reads and responds with a block of memory starting at the
# address set previously or the end of the last block operation (program/read).
s.write(struct.pack(">cH", 'g', len(CodeImage)))
assert(Fetch(len(CodeImage)) == CodeImage)

# 'E' - Exit bootloader and execute code from Flash. Responds with '\r'.
print "Exiting..."
s.write('E')
assert(Fetch(1) == '\r')

s.close()
