import os # for mkdir
import sys # for path tweaks

sys.path = ['..\..\..\python\pycrc'] + sys.path
import crcUtil

def hex_checksum(line):
    length = int(line[1:3], 16) + 4 # 1 byte length, 2 byte addr, 1 byte type
    bytes = [int(line[i*2+1:i*2+3], 16) for i in range(0, length)]
    return ((sum(bytes) ^ 0xFF) + 1) & 0xFF

# Like hex() but no leading 0x, always two digits, and always uppercase...
def niceHex(number):
    numHex = hex(number)
    numHex = numHex[2:] # chop off the leading '0x'
    if len(numHex) != 2:
        numHex = '0'+numHex # tack on a leading '0' if not two digits
    numHex = numHex.upper()
    return numHex

def intStr16(val):
    return '%c%c' % ((val>>8) & 0xFF, val & 0xFF)

def createLicenseStr(customerCode):
    SALT_MSB = 0x42
    SALT_LSB = 0x27

    saltedCustomerCode = customerCode
    saltedCustomerCode ^= (SALT_MSB << 8)
    saltedCustomerCode ^= SALT_LSB

    buf = intStr16(saltedCustomerCode)

    crc = crcUtil.getCrc(buf)

    licenseStr = "%04X%04X" % (customerCode, crc)
    return licenseStr
    
def signHex(sourceFullName, destFullName, licenseAddress, customerCode):
    """creates a signed HEX file from an unsigned one"""

    destDir = 'bin/'
    try:
        os.mkdir(destDir)
    except:
        # Dir already exists
        pass

    licenseStr = createLicenseStr(customerCode)
  
    sourceFileName = "bin/%s" % sourceFullName
    sourceFile = open(sourceFileName, 'r')
    
    destFileName = destDir + destFullName
    destFile = open(destFileName, 'w')
  
    specialHexRecFound = False
  
    for hexrec in sourceFile:
        if not specialHexRecFound:
            # Part 1 - We are looking for the record that requires the fixup
            # Is this the record that needs the fixup?
            if (hexrec[7:9] == '00') and (hexrec[3:7] == licenseAddress):
                specialHexRecFound = True
                adjustedHexRec = hexrec[0:9] + licenseStr + hexrec[17:len(hexrec)-3] # -3 to discard existing checksum and LF
                checksum = hex_checksum(adjustedHexRec)
                adjustedHexRec = adjustedHexRec + niceHex(checksum) +'\n'
                destFile.write(adjustedHexRec)
            else:
                destFile.write(hexrec)
        else:
            # Part 2 - Now we are just copying over the remainder of the file
            destFile.write(hexrec)

    sourceFile.close()
    destFile.close()
  
  
if __name__ == '__main__':  

    #checksum = hex_checksum(':09082700E2FB08E2FA08E2F922') # should be 02
    #checksum = hex_checksum(':09083000EBF208EAF208E9F222') # should be F9 (249)
    #checksum = hex_checksum(':00000001') # should be FF
    
    licenseAddress = '0954' # this varies from platform to platform
    SynapseId = 0
    
    # Builds without AES-128
    signHex('SnapZic2410.H00', 'signedSnapZic2410.H00', licenseAddress, SynapseId)
