import os # for mkdir
import sys # for path tweaks

sys.path = ['..\..\..\python\pycrc'] + sys.path
import crcUtil

def s19_checksum(line):
    length = int(line[2:4], 16)
    bytes = [int(line[i*2:i*2+2], 16) for i in range(1, length+1)]
    return ~sum(bytes) & 0xff

# Like hex() but no leading 0x, always two digits, and always uppercase...
def niceHex(number):
    numHex = hex(number)
    numHex = numHex[2:] # chop off the leading '0x'
    if len(numHex) != 2:
        numHex = '0'+numHex # tack on a leading '0' if not two digits
    numHex = numHex.upper()
    return numHex

def intStr16(val):
    return '%c%c' % ((val>>8) & 0xFF, val & 0xFF)

def createLicenseStr(customerCode):
    SALT_MSB = 0x42
    SALT_LSB = 0x27

    saltedCustomerCode = customerCode
    saltedCustomerCode ^= (SALT_MSB << 8)
    saltedCustomerCode ^= SALT_LSB

    buf = intStr16(saltedCustomerCode)

    crc = crcUtil.getCrc(buf)

    licenseStr = "%04X%04X" % (customerCode, crc)
    return licenseStr
    
def signS19(sourceShortName, destShortName, licenseAddress, customerCode):
    """creates a signed s19 file from an unsigned one"""

    destDir = 'bin/'
    try:
        os.mkdir(destDir)
    except:
        # Dir already exists
        pass

    licenseStr = createLicenseStr(customerCode)
  
    sourceFileName = "bin/%s.s19" % sourceShortName
    sourceFile = open(sourceFileName, 'r')
    
    destFileName = destDir + destShortName + '.s19'
    destFile = open(destFileName, 'w')
  
    specialSrecFound = False
  
    for srec in sourceFile:
        if not specialSrecFound:
            # Part 1 - We are looking for the record that requires the fixup
            # Is this the record that needs the fixup?
            if (srec[0:2] == 'S1') and (srec[4:8] == licenseAddress):
                specialSrecFound = True
                adjustedSrec = srec[0:8] + licenseStr + srec[16:len(srec)-3] # -3 to discard existing checksum and LF
                checksum = s19_checksum(adjustedSrec)
                adjustedSrec = adjustedSrec + niceHex(checksum) +'\n'
                destFile.write(adjustedSrec)
            else:
                destFile.write(srec)
        else:
            # Part 2 - Now we are just copying over the remainder of the file
            destFile.write(srec)

    sourceFile.close()
    destFile.close()
  
  
if __name__ == '__main__':  
  
    licenseAddress = '182C' # this varies from platform to platform
    SynapseId = 0
    
    # Builds without AES-128
    signS19('Snapper', 'signedSnapper',licenseAddress, SynapseId) # the original build (with extra error checks)
    #  signS19('Release', 'signedRelease',licenseAddress, SynapseId) # the new/leaner non-checking build
    # Builds with AES-128
    #  signS19('DebugAES128', 'signedDebugAES128',licenseAddress, SynapseId)     # the original build (with extra error checks)
    #  signS19('ReleaseAES128', 'signedReleaseAES128',licenseAddress, SynapseId) # the new/leaner non-checking build
