import PacketSerialProtocol
import RpcCodec
from serialwrapper import PyserialDriver
import time
import binascii
import PacketQueue


class MyPollSource:
  def __init__(self):
    self.registry = []

  def poll(self):
    for cb in self.registry:
      cb()

  def registerPollCallback(self, handler):
    self.registry.append(handler)


class PsdCaller:
  def __init__(self, num, psd):
    self.num = num
    self.psd = psd
    psd.registerCallback("tx", self.tx)
    psd.registerCallback("rx", self.rx)
    self.txQueue = []

  def rx(self,buf):
    pass
    #print "PSD %d: rx = %s" % (self.num, buf)

  def tx(self,status):
    print "PSD %d: tx status = %d" % (self.num, int(status))
    if self.txQueue:
      self.psd.send_data(self.txQueue.pop(0))

  def startTx(self, txQueue):
    self.txQueue = txQueue
    self.psd.send_data(self.txQueue.pop(0))

class CallSink:
  def __init__(self, num):
    self.num = num
    self.count = 0

  class CallProxy(object):
    def __init__(self, cs, name):
      self.fname = name
      self.cs = cs
    def __call__(self, *args):
      print "#%d CallSink%d (from %s) :  %s%s" % (self.cs.count, self.cs.num, binascii.hexlify(self.cs.remoteAddr), self.fname, args)

  def __getattr__(self, name):
    self.count += 1
    return CallSink.CallProxy(self, name)

#---------------------------------------------------------------------------------------------------
if __name__ == '__main__':
  import logging
  import copy
  logging.basicConfig(level=logging.DEBUG,
                      format='%(asctime)s:%(msecs)03d %(levelname)-8s %(name)-8s %(message)s',
                      datefmt='%H:%M:%S')


  print "Test PacketSerialDriver"

  pollSource = MyPollSource()

  drv = PyserialDriver.PyserialWrapper(outputType=PyserialDriver.PyserialWrapper.TYPE_PYSERIAL, port=21) #port=4)
  drv.serial.baudrate = 38400
  drv.serial.rtscts = 0

  psd = PacketSerialProtocol.PacketSerialDriver(drv, pollSource)

  psd.verboseLogging = True

  caller = PsdCaller(1, psd)

  # Queue up some commands and kick things off
  # cmdQueue = ["This is a test%d" % n for n in range(1,2)]
  # caller.startTx(cmdQueue)


  # Send an RPC call
  rpcCallSink = CallSink(1)
  queue = PacketQueue.Connector(rpcCallSink, psd)

  rpcDescr1 = RpcCodec.RpcDescriptor()
  rpcDescr1.srcAddr = '\x00\x00\x01'
  #rpcDescr1.dstAddr = '\x00\xDE\x03'
  #rpcDescr1.dstAddr = '\x00\xDE\x02'
  #rpcDescr1.dstAddr = '\x00\x17\xEC'
  rpcDescr1.dstAddr = '\x00\x1B\x5A'
  #rpcDescr1.funcName = "ping"
  rpcDescr1.funcName = "eraseImage"
  rpcDescr1.args = ()

  for i in range(1):
    rd = copy.copy(rpcDescr1)
    #rd.args = (i,)
    queue.rpcReady(rd)

  if False:
    rpcDescr2 = copy.copy(rpcDescr1)
    rpcDescr2.dstAddr = '\x00\xDE\x02'
    rpcDescr2.funcName = "setPinDir"
    rpcDescr2.args = (2,True)

    rpcDescr3 = copy.copy(rpcDescr1)
    rpcDescr3.dstAddr = '\x00\xDE\x02'
    rpcDescr3.funcName = "writeGpio"
    rpcDescr3.args = (2,True)

    queue.rpcReady(rpcDescr1)
    queue.rpcReady(rpcDescr2)
    queue.rpcReady(rpcDescr3)




# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  # Let the simulation begin!\
  while True:
    drv.readPoll()
    drv.writePoll()
    pollSource.poll()
    time.sleep(0.050)

